/**
 * @file
 * Defines Javascript behaviors for the web3_signer:metamask plugin form.
 */
(function (Drupal) {
  'use strict';

  /*
   * Request accounts from web3 wallet.
   */
  Drupal.AjaxCommands.prototype.walletCommand = async function (ajax, response, status) {
    // Avoid infinite loop, just in case.
    if (ajax.options.data._triggering_element_name == 'walletCommand') {
      return;
    }

    // @todo How to use another library web3js or ethers.js ?
    ethereum = typeof window.ethereum == 'object' ? window.ethereum : false;
    if (!ethereum) {
      console.log('Ethereum not enabled in this browser');
      return;
    }

    const messages = new Drupal.Message();

    // Handle calls to specific functions (e.g. `ethereum.network`).
    if (response.method.substr(0, 'ethereum'.length) == 'ethereum') {
      let results = Function("return " + response.method)()

       // Resend information to Drupal.
       ajax.options.data._triggering_element_name = 'walletCommand';
       ajax.options.data._triggering_element_value = {
         success: true,
         method: response.method,
         data: results
       };
       ajax.$form.ajaxSubmit(ajax.options);
       return;
    }

    await ethereum.request({
      method: response.method,
      params: response.params,
    })
      .then(function (results) {
        messages.clear();
        messages.add(
          Drupal.t('Method: @method', { '@method': response.method }) + '<br>' +
          Drupal.t('Result: @result', { '@result': Array.isArray(results) ? results.join('<br>') : results })
        );

        // Resend information to Drupal.
        ajax.options.data._triggering_element_name = 'walletCommand';
        ajax.options.data._triggering_element_value = {
          success: true,
          method: response.method,
          data: results
        };
        ajax.$form.ajaxSubmit(ajax.options);
      })
      .catch(function (error) {
        messages.clear();
        messages.add(
          Drupal.t('Code: @code', { '@code': error.code }) + '<br>' +
          Drupal.t('Message: @message', { '@message': error.message }),
          { type: 'error' }
        );
      });
  };

})(Drupal);
